import datetime
import textwrap
import os
import sys

import pytz

import git
from git.exc import GitCommandError
from git.objects.commit import Commit

def changelog_from_git(project, version_suffix, path,
        repository='eobuilder',
        maintainer_name='eobuilder',
        maintainer_email='eobuilder@entrouvert.com'):
    repo = git.repo.Repo(path)

    versions = []

    try:
        last_tag = repo.git.describe(abbrev=0)
    except GitCommandError:
        # no tag version is 0.0.<number of commits>
        versions.append(('0.0.%d' % len(repo.head.log()), repo.head.commit))
    else:
        if repo.tags[last_tag].commit != repo.head.commit:
            versions.append((repo.git.describe(), repo.head.commit))
        versions.append((last_tag, repo.tags[last_tag].commit))
        while True:
            try:
                last_tag = repo.git.describe(last_tag+'~', abbrev=0)
            except GitCommandError:
                break
            else:
                versions.append((last_tag, repo.tags[last_tag].commit))
    n = len(versions)
    for i in range(n):
        name, commit = versions[i]
        if i == n-1:
            logs = list(commit.iter_parents())
        else:
            reflog = '%s..%s' % (versions[i+1][1].binsha.encode('hex'), commit.binsha.encode('hex'))
            logs = list(Commit.iter_items(repo, reflog))
        if not logs:
            continue
        version = name.lstrip('v').replace('-', '.')
        if i == 0:
            version += version_suffix
        yield '%s (%s) %s; urgency=low' % (project, str(version), repository)
        yield ''
        for l in logs:
            try:
                t = '(%s) %s' % (l.hexsha[:6], l.summary)
            except LookupError as e:
                t = '(%s) %s' % (l.hexsha[:6], e)
            lines = textwrap.wrap(t, 80)
            for prefix, suffix in zip(['  * '] + ['    ']*(len(lines)-1), lines):
                yield (prefix + suffix).encode('utf-8')
        paris = pytz.timezone('Europe/Paris')
        date = datetime.datetime.fromtimestamp(logs[0].committed_date + logs[0].committer_tz_offset).replace(
                    tzinfo=pytz.UTC).astimezone(paris).strftime('%a, %d %b %Y %H:%M:%S %z')
        yield ''
        yield ' -- %s <%s>  %s' % (maintainer_name, maintainer_email, date)
        yield ''

if __name__ == '__main__':
    project = sys.argv[1]
    version_suffix = sys.argv[2]
    if len(sys.argv) > 3:
        path = sys.argv[3]
    else:
        path = os.getcwd()
    print '\n'.join(changelog_from_git(project, version_suffix, path))
